#ifndef INCLUDE__BasicBlockWidget_hxx
#define INCLUDE__BasicBlockWidget_hxx

#include "gui/qt.hxx"
#include "disassembler/BasicBlock.hxx"
#include <vector>
#include <cassert>

class BasicBlockWidget : public QGraphicsItem
{
public:
    BasicBlockWidget(const QString& name, BasicBlock * block);

	void addItem(uint8_t* bytes, size_t num_bytes, const QString& line);

    QRectF boundingRect() const  {
        qreal penWidth = 1;
		QRectF result(x - penWidth / 2, y - penWidth / 2,
                      dx + penWidth, dy + penWidth);


		if (next[0]) {
			if (next[1]) {
				result |= QRectF(QPointF(x + dx/3, y+dy),
								 mapFromScene(next[0]->getEntry()));
			} else {
				result |= QRectF(QPointF(x + dx/2, y+dy),
								 mapFromScene(next[0]->getEntry()));
			}
		}
		if (next[1])
			result |= QRectF(QPointF(x + 2*dx/3, y+dy),
							 mapFromScene(next[1]->getEntry()));

        return result;
    }
	void mouseMoveEvent(QGraphicsSceneMouseEvent *event) {
		QGraphicsItem::mouseMoveEvent(event);
		std::for_each(previous.begin(), previous.end(),
					  [&] (BasicBlockWidget * item) {
						  item->update(boundingRect() | item->boundingRect());
					  });
	}

	QPointF getEntry() const {
		return mapToScene(QPointF(x + dx/2, y));
	}

    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
               QWidget *widget) {
        painter->fillRect(x, y, dx, dy, QColor(0xcc, 0xcc, 0xff, 0xff));
        painter->setPen(QColor(0x00, 0x00, 0xff, 0xff));
        painter->drawRect(x, y, dx, dy);
        painter->drawText(0, -5, name);
		if (_widget.rowCount() != 0)
			_widget.render(painter);

		if (next[0]) {
			if (next[1]) {
				painter->setPen(QColor(0x00, 0xff, 0x00, 0xff));
				painter->drawLine(QPointF(x + dx/3, y+dy),
								  mapFromScene(next[0]->getEntry()));
			} else {
				painter->setPen(QColor(0x00, 0x00, 0x00, 0xff));
				painter->drawLine(QPointF(x + dx/2, y+dy),
								  mapFromScene(next[0]->getEntry()));
			}
		}
		painter->setPen(QColor(0xff, 0x00, 0x00, 0xff));
		if (next[1])
			painter->drawLine(QPointF(x + 2*dx/3, y+dy),
							  mapFromScene(next[1]->getEntry()));

    }

	void addPrevious(BasicBlockWidget * widget) {
		previous.push_back(widget);
	}

	void addNext(BasicBlockWidget * left, BasicBlockWidget * right) {
		next[0] = left;
		next[1] = right;
	}

private:
    int x, y, dx, dy;
    QTableWidget _widget;
	QString name;
	BasicBlock * block;
	std::vector<BasicBlockWidget*> previous;
	BasicBlockWidget* next[2];
};

#endif
